/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */



package com.openexchange.office.json.actions;

import com.openexchange.office.IExporter;
import com.openexchange.office.IImporter;
import com.openexchange.office.tools.DocumentFormat;
import com.openexchange.server.ServiceLookup;

public class DocumentFilterHelper {

	
    /**
     * Provides the importer for the given document format.
     *
     * @return The Importer for the given document format
     */
    static public IImporter getImporter(ServiceLookup services, DocumentFormat documentFormat) {
        IImporter importer = null;

        switch(documentFormat) {
            case DOCX: {
                importer = services.getService(com.openexchange.office.ooxml.docx.Importer.class);
                break;
            }

            case PPTX: {
                importer = services.getService(com.openexchange.office.ooxml.pptx.Importer.class);
                break;
            }

            case XLSX: {
                importer = services.getService(com.openexchange.office.ooxml.xlsx.Importer.class);
                break;
            }

            case ODT:
            case ODS: {
                importer = services.getService(com.openexchange.office.odf.Importer.class);
                break;
            }

            case NONE:
            case ODG:
            case ODP:
            default: {
                break;
            }
        }
        return importer;
    }

    /**
     * Provides the exporter for the given document format
     *
     * @return The Exporter for the given document format
     */
    static public IExporter getExporter(ServiceLookup services, DocumentFormat documentFormat) {
        IExporter exporter = null;

        switch(documentFormat) {

            case DOCX: {
                exporter = services.getService(com.openexchange.office.ooxml.docx.Exporter.class);
                break;
            }

            case PPTX: {
                exporter = services.getService(com.openexchange.office.ooxml.pptx.Exporter.class);
                break;
            }

            case XLSX: {
                exporter = services.getService(com.openexchange.office.ooxml.xlsx.Exporter.class);
                break;
            }

            case ODT:
            case ODS: {
                exporter = services.getService(com.openexchange.office.odf.Exporter.class);
                break;
            }

            case NONE:
            case ODG:
            case ODP:
            default: {
                break;
            }
        }
        return exporter;
    }
	
}

